%               Copyright (C) 2017 Joel Wraith

%This program is free software: you can redistribute it and/or modify it 
%under the terms of the GNU General Public License as published by the 
%Free Software Foundation, either version 3 of the License, or 
%(at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.


%                          CONFIG
    
%Set settings for the hologram, will be read in from XML file in v1.0
 bandpassFilter = false;
 bandpassFilterMin = 2;
 bandpassFilterMax = 40;
 steps = 150;
 wavelength = 0.66;
 medianRefractiveIndec = 1.3326;
 pixelsPerMicron = 4.29;
 stepSize = 1/pixelsPerMicron;
 offset = -300 * stepSize;
 constPWM = pixelsPerMicron * wavelength / medianRefractiveIndec;
 radiansPerPixel = -2 * pi * 1i * (1/constPWM);
 

%Load images into memory as double arrays
backgroundImage = im2double(imread('background.png'));
foregroundImage = im2double(imread('foreground.png'));
%Establish dimensions of read images
[rowCount, columnCount] = size(foregroundImage);

%Let all 0 pixels in the array be equal to one
backgroundImage(backgroundImage == 0) = 1;

%Establish the bandpass filter if the user selects
if bandpassFilter
    maximumDimension = max(rowCount, columnCount);
    halfPoint = maximumDimension / 2.0;
    maxValue = -(((bandpassFilterMax * 2.0)/maximumDimension)^2);
    minValue = -(((bandpassFilterMin * 2.0)/maximumDimension)^2);
    [ii,jj] = meshgrid(0:columnCount - 1);
    tempFilterArray = (ii - halfPoint).^2 + (jj - halfPoint).^2;
    filterArrayMax = exp(tempFilterArray * maxValue);
    filterArrayMin = exp(tempFilterArray * minValue);
    filterArray = filterArrayMax - filterArrayMin;
else %They didn't. Set the filter to a matrix of ones so that it has no effect on the final image
    filterArray = ones(rowCount, columnCount);
end

fourierTransformedArray = fftshift(fft2((foregroundImage ./ backgroundImage) - 1));

[ii,jj] = meshgrid(0:rowCount - 1, 0:columnCount - 1);
outputArray = ((ii - (rowCount / 2)) * (constPWM / rowCount)).^2 ...
            + ((jj - (columnCount/ 2)) * (constPWM / columnCount)).^2;
outputArray = sqrt(1 - outputArray) - 1;
imageStack = zeros(steps + 4,rowCount, columnCount);
 
stepIntervals = linspace(0, steps * stepSize, steps + 1);

for jj = 1:steps
    stackLayer = outputArray;
    
    dppm = stepIntervals(jj) * pixelsPerMicron;
    if dppm > 0
        stackLayer = conj(stackLayer);
    end
    
    radiansPerPixel * dppm;
    
    stackLayer = stackLayer .* (radiansPerPixel * dppm);
    stackLayer = filterArray .* exp(stackLayer) .* fourierTransformedArray;
    
%     postexp = stackLayer;    
%     jane = ifftshift(stackLayer);
%     hannah = ifft2(jane);
    
    stackLayer = real(ifft2(ifftshift(stackLayer)));
    
    bob = stackLayer;
    
    minElement = min(stackLayer(:));
    maxElement = max(stackLayer(:));
     
    l5 = stackLayer - minElement;
    l6 = 1 / (maxElement - minElement);
    stackLayer = l5 * l6;
    
    %writeVideo(v,stackLayer);
    if jj == 1
        imageStack(1,:,:) = stackLayer;
        imageStack(2,:,:) = stackLayer;
    end
    imageStack(jj + 2,:,:) = stackLayer;
    
    if jj == steps
        imageStack(jj+3,:,:) = stackLayer;
        imageStack(jj+4,:,:) = stackLayer;
    end  
end

v = VideoWriter('stack.avi');
open(v);
for jj = 1:steps
    writeVideo(v,squeeze(imageStack(jj,:,:)));
end
close(v)


sobelFilter = [0, -1, 0; 0, 0, 0; 0, 1, 0];
imageStack = permute(imageStack, [2 3 1]);
convolvedArray = zeros(rowCount, steps + 6, columnCount + 2);

for jj = 1:rowCount %3.3 seconds
    convolvedArray(jj,:,:) = conv2(transpose(squeeze(imageStack(jj, :, :))), sobelFilter); 
    %convolvedArray(jj,:,:) = edge(transpose(squeeze(imageStack(jj, :, :))), 'Canny'); 
end

temp = zeros(columnCount, rowCount, steps + 6);
convolvedArray = permute(convolvedArray, [3 2 1]);

for jj = 1:rowCount %3.7 seconds
    temp(jj,:,:) = transpose(squeeze(convolvedArray(jj + 1,:,:))); 
end
convolvedArray = permute(temp, [3 2 1]);
temp = zeros(steps, rowCount, columnCount);

for jj = 1:steps
    temp(jj,:,:) = convolvedArray(jj + 3,:,:);
end

convolvedArray = temp;
convolvedArray(convolvedArray < 0) = 0;
convolvedArray = convolvedArray .* (1/max(convolvedArray(:)));

l1 = convolvedArray;
v = VideoWriter('grad.avi');
open(v)
for jj = 1:steps
    writeVideo(v,squeeze(convolvedArray(jj,:,:)));
end
imageValues = convolvedArray .* 255;
gradiantImage = uint8(squeeze(max(imageValues,[],1)));
imwrite(gradiantImage, 'gradimg.png');

mathify = @(aMatrix)double(log(mean(exp(double(aMatrix(:))))));

newPixelSize = 16;

for ii = 0:rowCount/newPixelSize - 1
    yCoord = (ii * newPixelSize) + 1;
    SOMESTAT =  mathify(gradiantImage(yCoord:yCoord+(newPixelSize-1),1:newPixelSize));
    
    newRow = ones(newPixelSize) .* SOMESTAT;
    for jj = 1:(rowCount/newPixelSize)-1
        xCoord = (jj * newPixelSize) + 1;
        SOMESTAT = mathify(gradiantImage(yCoord:yCoord+(newPixelSize-1),xCoord:xCoord + (newPixelSize-1)));
        medianBlock = ones(newPixelSize) * mathify(gradiantImage(yCoord:yCoord+3,xCoord:xCoord + (newPixelSize-1)));
        newRow = cat(2,newRow, medianBlock);
    end
    if ii == 0
        blockMatrix = newRow;
    else
        
        blockMatrix = cat(1, blockMatrix, newRow);
    end
end


close(v)
