%               Copyright (C) 2017 Joel Wraith

%This program is free software: you can redistribute it and/or modify it 
%under the terms of the GNU General Public License as published by the 
%Free Software Foundation, either version 3 of the License, or 
%(at your option) any later version.

%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.

%You should have received a copy of the GNU General Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

gradientImage = imread('gradimg.png');
imshow(gradientImage);
mathify = @(aMatrix)double(log(mean(exp(double(aMatrix(:)))))) ;
newPixelSize = 8;
rowCount = 512;

for ii = 0:rowCount/newPixelSize - 1
    yCoord = (ii * newPixelSize) + 1;
    SOMESTAT =  mathify(gradientImage(yCoord:yCoord+(newPixelSize-1),1:newPixelSize));
    
    newRow = ones(newPixelSize) .* SOMESTAT;
    for jj = 1:(rowCount/newPixelSize)-1
        xCoord = (jj * newPixelSize) + 1;
        SOMESTAT = mathify(gradientImage(yCoord:yCoord+(newPixelSize-1),xCoord:xCoord + (newPixelSize-1)));
        medianBlock = ones(newPixelSize) * mathify(gradientImage(yCoord:yCoord+3,xCoord:xCoord + (newPixelSize-1)));
        newRow = cat(2,newRow, medianBlock);
    end
    if ii == 0
        blockMatrix = newRow;
    else
        blockMatrix = cat(1, blockMatrix, newRow);
    end
end

for ii = 1:rowCount/newPixelSize - 1
    yCoord = (ii * newPixelSize) - 1;
    SOMESTAT =  mathify(gradientImage(yCoord:yCoord+(newPixelSize-1),1:newPixelSize));
    
    newRow = ones(newPixelSize) .* SOMESTAT;
    for jj = 1:(rowCount/newPixelSize) - 2
        xCoord = (jj * newPixelSize) - 1;
        SOMESTAT = mathify(gradientImage(yCoord:yCoord+(newPixelSize-1),xCoord:xCoord + (newPixelSize-1)));
        medianBlock = ones(newPixelSize) * mathify(gradientImage(yCoord:yCoord+3,xCoord:xCoord + (newPixelSize-1)));
        newRow = cat(2,newRow, medianBlock);
    end
    if ii == 1
        blockMatrix2 = newRow;
    else
        blockMatrix2 = cat(1, blockMatrix2, newRow);
    end
end

%%Pixelate and detect edges
blockMatrix = uint8(blockMatrix);
maxE = max(blockMatrix(:));
minE = min(blockMatrix(:));
distributionFactor = 255/(maxE - minE);  
blockMatrix = (blockMatrix - minE) .* distributionFactor;

blockMatrix2 = uint8(blockMatrix2);
maxE = max(blockMatrix2(:));
minE = min(blockMatrix2(:));
distributionFactor = 255/(maxE - minE);  
blockMatrix2 = (blockMatrix2 - minE) .* distributionFactor;

allPixelBlocks = cat(1, blockMatrix(:), blockMatrix2(:));
% x = prctile(allPixelBlocks,95);
% blockMatrix(blockMatrix < x) = 0;
% blockMatrix2(blockMatrix2 < x) = 0;

padding = zeros(rowCount);
padding((newPixelSize / 2) + 1:rowCount - (newPixelSize / 2),(newPixelSize / 2) + 1:rowCount - (newPixelSize / 2)) = blockMatrix2;
blockMatrix2 = uint8(padding);

threasholdValue = 0.1;

CannyFilter = uint8(or(edge(blockMatrix, 'Canny', threasholdValue), edge(blockMatrix2, 'Canny', threasholdValue)));

getNeighbourGrid = @()cat(2,reshape(meshgrid(-1:1,-1:1),[9,1]), reshape(transpose(meshgrid(-1:1,-1:1)),[9,1]));

coords = int16(cat(3,meshgrid(1:512,1:512),transpose(meshgrid(1:512,1:512))));
workingList = reshape(cat(3, int16(CannyFilter), coords), [512*512 3 1]);
workingList(find(workingList(:,1)==0),:) = [];


dectectedImage = zeros(512, 512, 3);
filterBackground = gradientImage;
for ii = 1:512
    for jj = 1:512
        if CannyFilter(ii,jj) == 0
            dectectedImage(ii,jj,1:3) = [filterBackground(ii,jj) filterBackground(ii,jj) filterBackground(ii,jj)];
            %dectectedImage(ii,jj,1:3) = [0 0 0];
        else
            dectectedImage(ii,jj,1:3) = [255 0 0];
        end
    end
end

newImage = cat(3,gradientImage, gradientImage, gradientImage);

[centres, radii] = imfindcircles(gradientImage,[15, 35],'Sensitivity',0.95);

viscircles(centres, radii,'EdgeColor', 'b');

circleData = cat(2, centres, radii);

[count, dim] = size(circleData);

size(newImage)

for ii=1:count
    x = int16(circleData(ii,1));
    y = int16(circleData(ii,2));
    r = circleData(ii,3);

    thetaValues = linspace(0,2*pi,int16(2 * r * pi));
    X  = double(x) + r * cos(thetaValues);
    Y = double(y) + r * sin(thetaValues);
    tuple = transpose(cat(1,int16(X),int16(Y)));
    tuple(any(tuple <= 0,2),:) = [];
    tuple(any(tuple > 512,2),:) = [];
    [dataCount, dim] = size(tuple);
    for pixelCount = 1:dataCount
        xx = tuple(pixelCount,1);
        yy = tuple(pixelCount,2);

        newImage(yy,xx,:) = [255 255 0];
    end
end

imwrite(uint8(dectectedImage), 'detected.png');
imwrite(uint8(newImage), 'circles.png');
imwrite(blockMatrix,'pixelated.png');
imwrite(blockMatrix2,'pixelated2.png');